#include <math.h>

/* Cumulative Normal distribution from Hull */
double cumNorm(double x){
  const double a1 = 0.319381530,
               a2 =-0.356563782,
               a3 = 1.781477937,
               a4 =-1.821255978,
               a5 = 1.330274429,
               gmma = 0.2316419;
  double k,nprime;

  if(x<0){
    return(1 - cumNorm(-x));
  }else{
    k = 1.0/(1.0 + gmma*x);
    nprime = exp(-x*x/2.0)/sqrt(2*M_PI);
    return(1 - nprime*( a1*k
		       +a2*k*k
		       +a3*k*k*k
		       +a4*k*k*k*k
		       +a5*k*k*k*k*k));
  }
}

/* Black-Scholes european call pricing with continuous dividends from Tucker
 *
 * S = current price
 * X = strike price
 * r = risk-free rate
 * s = sigma = standard deviation
 * T = periods to maturity
 * d = delta = dividend rate
 */

#define N(x) cumNorm((x));

double euroCall(double S, double X, double r, double s, double T, double d){
  double d3,Sd;
  double n1,n2;

  Sd = S*exp(-d*T);
  d3 = (log(Sd/X) + (r + (s*s/2))*T)/(s*sqrt(T));
  n1 = N(d3);
  n2 = N(d3 - s*sqrt(T));

  return( Sd*n1 - X*exp(-r*T)*n2 );
}

